@@ -46,15 +46,16 @@
 
 #include <config.h>
 
+#include "auth.h"
 #include "dav_db.h"
 #include "strarray.h"
+#include "util.h"
 #include "vparse.h"
 
 struct carddav_db;
 
-#define CARDDAV_CREATE 0x01
-#define CARDDAV_TRUNC  0x02
-
+#define CARDDAV_KIND_CONTACT 0
+#define CARDDAV_KIND_GROUP 1
 struct carddav_data {
     struct dav_data dav;  /* MUST be first so we can typecast */
     unsigned version;
@@ -63,10 +64,13 @@ struct carddav_data {
     const char *fullname;
     const char *name;
     const char *nickname;
-    strarray_t emails;
-    strarray_t member_uids;
+    strarray_t *emails;
+    strarray_t *member_uids;
 };
 
+typedef int carddav_cb_t(void *rock, struct carddav_data *cdata);
+
+
 /* prepare for carddav operations in this process */
 int carddav_init(void);
 
@@ -74,8 +78,8 @@ int carddav_init(void);
 int carddav_done(void);
 
 /* get a database handle corresponding to mailbox */
-struct carddav_db *carddav_open_mailbox(struct mailbox *mailbox, int flags);
-struct carddav_db *carddav_open_userid(const char *userid, int flags);
+struct carddav_db *carddav_open_mailbox(struct mailbox *mailbox);
+struct carddav_db *carddav_open_userid(const char *userid);
 
 /* close this handle */
 int carddav_close(struct carddav_db *carddavdb);
@@ -83,32 +87,57 @@ int carddav_close(struct carddav_db *carddavdb);
 /* lookup an entry from 'carddavdb' by resource
    (optionally inside a transaction for updates) */
 int carddav_lookup_resource(struct carddav_db *carddavdb,
-			   const char *mailbox, const char *resource,
-			   int lock, struct carddav_data **result);
+                           const char *mailbox, const char *resource,
+                           struct carddav_data **result,
+                           int tombstones);
 
 /* lookup an entry from 'carddavdb' by iCal UID
    (optionally inside a transaction for updates) */
 int carddav_lookup_uid(struct carddav_db *carddavdb, const char *ical_uid,
-		      int lock, struct carddav_data **result);
+                       struct carddav_data **result);
 
-/* check if an email address exists on any card */
-int carddav_getemail(struct carddav_db *carddavdb, const char *key);
-strarray_t *carddav_getgroup(struct carddav_db *carddavdb, const char *key);
+/* check if an email address exists on any card.
+   returns the groups its in (if any) */
+strarray_t *carddav_getemail(struct carddav_db *carddavdb, const char *key);
+strarray_t *carddav_getemail2uids(struct carddav_db *carddavdb, const char *key,
+                                  const char *mboxname);
+strarray_t *carddav_getuid2groups(struct carddav_db *carddavdb, const char *key,
+                                  const char *mboxname, const char *otheruser);
+
+/* checks if a group exists (by id).
+   returns emails of its members (if any) */
+strarray_t *carddav_getgroup(struct carddav_db *carddavdb, const char *mailbox, const char *group);
+
+/* get a list of groups the given uid is a member of */
+strarray_t *carddav_getuid_groups(struct carddav_db *carddavdb, const char *uid);
+
+/* process each entry of type 'kind' for 'mailbox' in 'carddavdb' with cb() */
+int carddav_get_cards(struct carddav_db *carddavdb,
+                      const char *mailbox, const char *vcard_uid, int kind,
+                      carddav_cb_t *cb, void *rock);
+
+/* process each entry of type 'kind' and updated since 'oldmodseq'
+   in 'carddavdb' with cb() */
+int carddav_get_updates(struct carddav_db *carddavdb,
+                        modseq_t oldmodseq, const char *mboxname, int kind,
+                        carddav_cb_t *cb, void *rock);
 
 /* process each entry for 'mailbox' in 'carddavdb' with cb() */
 int carddav_foreach(struct carddav_db *carddavdb, const char *mailbox,
-		   int (*cb)(void *rock, void *data),
-		   void *rock);
+                    carddav_cb_t *cb, void *rock);
 
 /* write an entry to 'carddavdb' */
-int carddav_write(struct carddav_db *carddavdb, struct carddav_data *cdata,
-		 int commit);
+int carddav_write(struct carddav_db *carddavdb, struct carddav_data *cdata);
+
+/* write an entry form a vcard */
+int carddav_writecard(struct carddav_db *carddavdb, struct carddav_data *cdata,
+                      struct vparse_card *vcard);
 
 /* delete an entry from 'carddavdb' */
-int carddav_delete(struct carddav_db *carddavdb, unsigned rowid, int commit);
+int carddav_delete(struct carddav_db *carddavdb, unsigned rowid);
 
 /* delete all entries for 'mailbox' from 'carddavdb' */
-int carddav_delmbox(struct carddav_db *carddavdb, const char *mailbox, int commit);
+int carddav_delmbox(struct carddav_db *carddavdb, const char *mailbox);
 
 /* begin transaction */
 int carddav_begin(struct carddav_db *carddavdb);
@@ -119,10 +148,19 @@ int carddav_commit(struct carddav_db *carddavdb);
 /* abort transaction */
 int carddav_abort(struct carddav_db *carddavdb);
 
-/* create carddav_data from vparse_card */
-void carddav_make_entry(struct vparse_card *vcard, struct carddav_data *cdata);
+/* store a vcard to mailbox/resource */
+int carddav_store(struct mailbox *mailbox, struct vparse_card *vcard,
+                  const char *resource,
+                  strarray_t *flags, struct entryattlist *annots,
+                  const char *userid, struct auth_state *authstate,
+                  int ignorequota);
 
-/* release memory used by struct carddav_data */
-void carddav_data_fini(struct carddav_data *cdata);
+/* delete a carddav entry */
+int carddav_remove(struct mailbox *mailbox,
+                   uint32_t olduid, int isreplace,
+                   const char *userid);
+
+/* calculate a mailbox name */
+char *carddav_mboxname(const char *userid, const char *name);
 
 #endif /* CARDDAV_DB_H */
