@@ -62,7 +62,7 @@
 static void usage(void)
 {
     fprintf(stderr,
-	    "ctl_deliver [-C <altconfig>] -d [-f <dbfile>]\n");
+            "ctl_deliver [-C <altconfig>] -d [-f <dbfile>]\n");
     exit(-1);
 }
 
@@ -76,80 +76,80 @@ int main(int argc, char *argv[])
     enum { DUMP, PRUNE, NONE } op = NONE;
 
     if ((geteuid()) == 0 && (become_cyrus(/*is_master*/0) != 0)) {
-	fatal("must run as the Cyrus user", EC_USAGE);
+        fatal("must run as the Cyrus user", EC_USAGE);
     }
 
     while ((opt = getopt(argc, argv, "C:drE:f:")) != EOF) {
-	switch (opt) {
-	case 'C': /* alt config file */
-	    alt_config = optarg;
-	    break;
+        switch (opt) {
+        case 'C': /* alt config file */
+            alt_config = optarg;
+            break;
 
-	case 'd':
-	    if (op == NONE) op = DUMP;
-	    else usage();
-	    break;
+        case 'd':
+            if (op == NONE) op = DUMP;
+            else usage();
+            break;
 
         case 'f':
             if (alt_file == NULL) alt_file = optarg;
             else usage ();
             break;
 
-	case 'E':
-	    if (op == NONE) op = PRUNE;
-	    else usage();
-	    /* deprecated, but we still support it */
-	    days = optarg;
-	    break;
-	
-	default:
-	    usage();
-	    break;
-	}
+        case 'E':
+            if (op == NONE) op = PRUNE;
+            else usage();
+            /* deprecated, but we still support it */
+            days = optarg;
+            break;
+
+        default:
+            usage();
+            break;
+        }
     }
 
     switch (op) {
     case PRUNE: {
-	char buf[4096];
+        char buf[4096];
 
-	fprintf(stderr, "ctl_deliver -E is deprecated: "
-		"using cyr_expire -E instead\n");
+        fprintf(stderr, "ctl_deliver -E is deprecated: "
+                "using cyr_expire -E instead\n");
 
-	r = snprintf(buf, sizeof(buf), "%s/cyr_expire", SERVICE_DIR);
-	if(r < 0 || r >= (int) sizeof(buf)) {
-	    fatal("cyr_expire command buffer not sufficiently big", EC_CONFIG);
-	}
+        r = snprintf(buf, sizeof(buf), "%s/cyr_expire", SBIN_DIR);
+        if(r < 0 || r >= (int) sizeof(buf)) {
+            fatal("cyr_expire command buffer not sufficiently big", EC_CONFIG);
+        }
 
-	if (alt_config)
-	    execl(buf, buf, "-C", alt_config, "-E", days, NULL);
-	else
-	    execl(buf, buf, "-E", days, NULL);
+        if (alt_config)
+            execl(buf, buf, "-C", alt_config, "-E", days, NULL);
+        else
+            execl(buf, buf, "-E", days, NULL);
 
-	break;
+        break;
     }
 
     case DUMP:
-	cyrus_init(alt_config, "ctl_deliver", 0, 0);
+        cyrus_init(alt_config, "ctl_deliver", 0, 0);
 
-	if (duplicate_init(alt_file) != 0) {
-	    fprintf(stderr, 
-		    "ctl_deliver: unable to init duplicate delivery database\n");
-	    exit(1);
-	}
+        if (duplicate_init(alt_file) != 0) {
+            fprintf(stderr,
+                    "ctl_deliver: unable to init duplicate delivery database\n");
+            exit(1);
+        }
 
-	printf("it is NOW: %d\n", (int) time(NULL));
-	printf("got %d entries\n", duplicate_dump(stdout));
+        printf("it is NOW: %d\n", (int) time(NULL));
+        printf("got %d entries\n", duplicate_dump(stdout));
 
-	r = 0;
+        r = 0;
 
-	duplicate_done();
-	cyrus_done();
-	break;
+        duplicate_done();
+        cyrus_done();
+        break;
 
     case NONE:
-	r = 2;
-	usage();
-	break;
+        r = 2;
+        usage();
+        break;
     }
 
     return r;
