@@ -70,30 +70,24 @@ static void usage(void)
 }
 
 /* Callback for use by delete_seen */
-static int deluserseen(void *rock __attribute__((unused)),
-		       const char *key,
-		       size_t keylen,
-		       const char *val __attribute__((unused)),
-		       size_t vallen  __attribute__((unused)))
+static int deluserseen(const mbentry_t *mbentry, void *rock __attribute__((unused)))
 {
-    char *name = xstrndup(key, keylen);
     struct mailbox *mailbox = NULL;
-    const char *userid;
     int r = 0;
 
-    r = mailbox_open_irl(name, &mailbox);
+    r = mailbox_open_irl(mbentry->name, &mailbox);
     if (r) goto done;
 
-    userid = mboxname_to_userid(name);
+    char *userid = mboxname_to_userid(mbentry->name);
     if (userid) {
-	printf("removing seen for %s on %s\n", userid, mailbox->name);
-	if (do_remove) seen_delete_mailbox(userid, mailbox);
+        printf("removing seen for %s on %s\n", userid, mailbox->name);
+        if (do_remove) seen_delete_mailbox(userid, mailbox);
+        free(userid);
     }
 
     mailbox_close(&mailbox);
 
 done:
-    free(name);
     return r;
 }
 
@@ -104,23 +98,23 @@ int main(int argc, char *argv[])
     char *alt_config = NULL;
 
     if ((geteuid()) == 0 && (become_cyrus(/*is_master*/0) != 0)) {
-	fatal("must run as the Cyrus user", EC_USAGE);
+        fatal("must run as the Cyrus user", EC_USAGE);
     }
 
     while ((opt = getopt(argc, argv, "C:d")) != EOF) {
-	switch (opt) {
-	case 'C': /* alt config file */
-	    alt_config = optarg;
-	    break;
+        switch (opt) {
+        case 'C': /* alt config file */
+            alt_config = optarg;
+            break;
 
-	case 'd':
-	    do_remove = 1;
-	    break;
+        case 'd':
+            do_remove = 1;
+            break;
 
-	default:
-	    usage();
-	    break;
-	}
+        default:
+            usage();
+            break;
+        }
     }
 
     cyrus_init(alt_config, "cyr_userseen", 0, 0);
