@@ -54,7 +54,6 @@
 #include "exitcodes.h"
 #include "global.h"
 #include "index.h"
-#include "imap/imap_err.h"
 #include "imapurl.h"
 #include "mailbox.h"
 #include "mboxlist.h"
@@ -62,17 +61,16 @@
 #include "xmalloc.h"
 #include "xstrlcpy.h"
 
+/* generated headers are not necessarily in current directory */
+#include "imap/imap_err.h"
 
 static int verbose = 0;
 
-static int dump_me(char *name, int matchlen, int maycreate, void *rock);
-static void print_seq(const char *tag, const char *attrib, 
-		      unsigned *seq, int n);
+static int dump_me(struct findall_data *data, void *rock);
+static void print_seq(const char *tag, const char *attrib,
+                      unsigned *seq, int n);
 static int usage(const char *name);
 
-/* current namespace */
-static struct namespace dump_namespace;
-
 struct incremental_record {
     unsigned incruid;
 };
@@ -80,61 +78,55 @@ struct incremental_record {
 int main(int argc, char *argv[])
 {
     int option;
-    char buf[MAX_MAILBOX_PATH+1];
-    int i, r;
+    int i;
     char *alt_config = NULL;
     struct incremental_record irec;
 
     if ((geteuid()) == 0 && (become_cyrus(/*is_master*/0) != 0)) {
-	fatal("must run as the Cyrus user", EC_USAGE);
+        fatal("must run as the Cyrus user", EC_USAGE);
     }
 
     while ((option = getopt(argc, argv, "vC:")) != EOF) {
-	switch (option) {
-	case 'v':
-	    verbose++;
-	    break;
+        switch (option) {
+        case 'v':
+            verbose++;
+            break;
 
-	case 'C': /* alt config file */
-	    alt_config = optarg;
-	    break;
+        case 'C': /* alt config file */
+            alt_config = optarg;
+            break;
 
-	default:
-	    usage(argv[0]);
-	    break;
-	}
+        default:
+            usage(argv[0]);
+            break;
+        }
     }
 
     if (optind == argc) {
-	usage(argv[0]);
+        usage(argv[0]);
     }
 
     cyrus_init(alt_config, "dump", 0, CONFIG_NEED_PARTITION_DATA);
     mboxlist_init(0);
     mboxlist_open(NULL);
 
-    /* Set namespace -- force standard (internal) */
-    if ((r = mboxname_init_namespace(&dump_namespace, 1)) != 0) {
-	syslog(LOG_ERR, "%s", error_message(r));
-	fatal(error_message(r), EC_CONFIG);
-    }
+    search_attr_init();
 
     irec.incruid = 0;
+    strarray_t *array = strarray_new();
     for (i = optind; i < argc; i++) {
-	strlcpy(buf, argv[i], sizeof(buf));
-	/* Translate any separators in mailboxname */
-	mboxname_hiersep_tointernal(&dump_namespace, buf,
-				    config_virtdomains ?
-				    strcspn(buf, "@") : 0);
-	(*dump_namespace.mboxlist_findall)(&dump_namespace, buf, 1, 0, 0,
-					   dump_me, &irec);
+        strarray_append(array, argv[i]);
     }
+    if (array->count)
+        mboxlist_findallmulti(NULL, array, 1, 0, 0, dump_me, &irec);
+
+    strarray_free(array);
 
     mboxlist_close();
     mboxlist_done();
 
     cyrus_done();
-    
+
     return 0;
 }
 
@@ -148,13 +140,20 @@ static int usage(const char *name)
 static void generate_boundary(char *boundary, size_t size)
 {
     assert(size >= 100);
-    
-    snprintf(boundary, size, "dump-%ld-%ld-%ld", 
-	     (long) getpid(), (long) time(NULL), (long) rand());
+
+    snprintf(boundary, size, "dump-%ld-%ld-%ld",
+             (long) getpid(), (long) time(NULL), (long) rand());
 }
 
-static int dump_me(char *name, int matchlen __attribute__((unused)),
-		   int maycreate __attribute__((unused)), void *rock)
+static search_expr_t *systemflag_match(int flag)
+{
+    search_expr_t *e = search_expr_new(NULL, SEOP_MATCH);
+    e->attr = search_attr_find("systemflags");
+    e->value.u = flag;
+    return e;
+}
+
+static int dump_me(struct findall_data *data, void *rock)
 {
     int r;
     char boundary[128];
@@ -163,16 +162,22 @@ static int dump_me(char *name, int matchlen __attribute__((unused)),
     struct incremental_record *irec = (struct incremental_record *) rock;
     struct searchargs searchargs;
     struct index_state *state;
-    unsigned *uids;
-    unsigned *uidseq;
+    unsigned *uids = NULL;
+    unsigned *uidseq = NULL;
     int i, n, numuids;
+    unsigned msgno;
+
+    /* don't want partial matches */
+    if (!data || !data->mbname) return 0;
+
+    const char *name = mbname_intname(data->mbname);
 
     r = index_open(name, NULL, &state);
     if (r) {
-	if (verbose) {
-	    printf("error opening %s: %s\n", name, error_message(r));
-	}
-	return 0;
+        if (verbose) {
+            printf("error opening %s: %s\n", name, error_message(r));
+        }
+        return 0;
     }
 
     generate_boundary(boundary, sizeof(boundary));
@@ -195,29 +200,39 @@ static int dump_me(char *name, int matchlen __attribute__((unused)),
     printf("\n");
 
     memset(&searchargs, 0, sizeof(struct searchargs));
+    searchargs.root = search_expr_new(NULL, SEOP_TRUE);
     numuids = index_getuidsequence(state, &searchargs, &uids);
+    search_expr_free(searchargs.root);
     print_seq("uidlist", NULL, uids, numuids);
     printf("\n");
 
     printf("  <flags>\n");
 
-    searchargs.system_flags_set = FLAG_ANSWERED;
+    searchargs.root = systemflag_match(FLAG_ANSWERED);
+    uidseq = NULL;
     n = index_getuidsequence(state, &searchargs, &uidseq);
+    search_expr_free(searchargs.root);
     print_seq("flag", "name=\"\\Answered\" user=\"*\"", uidseq, n);
     if (uidseq) free(uidseq);
 
-    searchargs.system_flags_set = FLAG_DELETED;
+    searchargs.root = systemflag_match(FLAG_DELETED);
+    uidseq = NULL;
     n = index_getuidsequence(state, &searchargs, &uidseq);
+    search_expr_free(searchargs.root);
     print_seq("flag", "name=\"\\Deleted\" user=\"*\"", uidseq, n);
     if (uidseq) free(uidseq);
 
-    searchargs.system_flags_set = FLAG_DRAFT;
+    searchargs.root = systemflag_match(FLAG_DRAFT);
+    uidseq = NULL;
     n = index_getuidsequence(state, &searchargs, &uidseq);
+    search_expr_free(searchargs.root);
     print_seq("flag", "name=\"\\Draft\" user=\"*\"", uidseq, n);
     if (uidseq) free(uidseq);
 
-    searchargs.system_flags_set = FLAG_FLAGGED;
+    searchargs.root = systemflag_match(FLAG_FLAGGED);
+    uidseq = NULL;
     n = index_getuidsequence(state, &searchargs, &uidseq);
+    search_expr_free(searchargs.root);
     print_seq("flag", "name=\"\\Flagged\" user=\"*\"", uidseq, n);
     if (uidseq) free(uidseq);
 
@@ -225,48 +240,67 @@ static int dump_me(char *name, int matchlen __attribute__((unused)),
 
     printf("</imapdump>\n");
 
-    for (i = 0; i < numuids; i++) {
-	const char *base;
-	size_t len;
+    i = 0;
+    while (i < numuids && uids[i] < irec->incruid) {
+        /* already dumped this message */
+        /* xxx could do binary search to get to the first
+           undumped uid */
+        i++;
+    }
 
-	if (uids[i] < irec->incruid) {
-	    /* already dumped this message */
-	    /* xxx could do binary search to get to the first
-	       undumped uid */
-	    continue;
-	}
+    for (msgno = 1; msgno <= state->exists; msgno++) {
+        struct buf buf = BUF_INITIALIZER;
+        struct index_map *im = &state->map[msgno-1];
+        struct index_record record;
 
-	printf("\n--%s\n", boundary);
-	printf("Content-Type: message/rfc822\n");
-	printf("Content-ID: %d\n", uids[i]);
-	printf("\n");
-	r = mailbox_map_message(state->mailbox, uids[i], &base, &len);
-	if (r) {
-	    if (verbose) {
-		printf("error mapping message %d: %s\n", uids[i], 
-		       error_message(r));
-	    }
-	    break;
-	}
-	fwrite(base, 1, len, stdout);
-	mailbox_unmap_message(state->mailbox, uids[i], &base, &len);
+        while (im->uid > uids[i] && i < numuids)
+            i++;
+        if (i >= numuids)
+            break;
+
+        if (im->uid < uids[i])
+            continue;
+
+        /* got a match */
+        i++;
+        memset(&record, 0, sizeof(struct index_record));
+        record.recno = im->recno;
+        record.uid = im->uid;
+        if (mailbox_reload_index_record(state->mailbox, &record))
+            continue;
+
+        printf("\n--%s\n", boundary);
+        printf("Content-Type: message/rfc822\n");
+        printf("Content-ID: %d\n", uids[i]);
+        printf("\n");
+        r = mailbox_map_record(state->mailbox, &record, &buf);
+        if (r) {
+            if (verbose) {
+                printf("error mapping message %u: %s\n", record.uid,
+                       error_message(r));
+            }
+            break;
+        }
+        fwrite(buf.s, 1, buf.len, stdout);
+        buf_free(&buf);
     }
 
     printf("\n--%s--\n", boundary);
 
+    free(uids);
     index_close(&state);
 
     return 0;
 }
 
 static void print_seq(const char *tag, const char *attrib,
-		      unsigned *seq, int n)
+                      unsigned *seq, int n)
 {
     int i;
 
     printf("  <%s%s%s>", tag, attrib ? " " : "", attrib ? attrib : "");
     for (i = 0; i < n; i++) {
-	printf("%u ", seq[i]);
+        printf("%u ", seq[i]);
     }
     printf("</%s>\n", tag);
 }
