@@ -64,7 +64,6 @@
 #include "mboxlist.h"
 #include "global.h"
 #include "exitcodes.h"
-#include "imap/imap_err.h"
 #include "mailbox.h"
 #include "map.h"
 #include "mboxname.h"
@@ -72,9 +71,12 @@
 #include "seen.h"
 #include "xmalloc.h"
 
-#define REQ_OK		0
-#define REQ_DENY	1
-#define REQ_UNK		2
+/* generated headers are not necessarily in current directory */
+#include "imap/imap_err.h"
+
+#define REQ_OK          0
+#define REQ_DENY        1
+#define REQ_UNK         2
 
 extern int optind;
 
@@ -86,16 +88,16 @@ static struct namespace fud_namespace;
 const int config_need_data = 0;
 
 static int handle_request(const char *who, const char *name,
-			  struct sockaddr *sfrom, socklen_t sfromsiz);
+                          struct sockaddr *sfrom, socklen_t sfromsiz);
 
 static void send_reply(struct sockaddr *sfrom, socklen_t sfromsiz, int status,
-		const char *user, const char *mbox,
-		int numrecent, time_t lastread, time_t lastarrived);
+                const char *user, const char *mbox,
+                int numrecent, time_t lastread, time_t lastarrived);
 
 static int soc = 0; /* inetd (master) has handed us the port as stdin */
 
-#define MAXLOGNAME 16		/* should find out for real */
-#define MAXDOMNAME 20		/* should find out for real */
+#define MAXLOGNAME 16           /* should find out for real */
+#define MAXDOMNAME 20           /* should find out for real */
 
 static int begin_handling(void)
 {
@@ -107,22 +109,22 @@ static int begin_handling(void)
     int r;
 
     while(1) {
-	if (signals_poll() == SIGHUP) {
-	    /* caught a SIGHUP, return */
-	    return 0;
-	}
+        if (signals_poll() == SIGHUP) {
+            /* caught a SIGHUP, return */
+            return 0;
+        }
 
-	memset(buf, 0, sizeof(buf));
+        memset(buf, 0, sizeof(buf));
 
-	sfromsiz = sizeof(struct sockaddr_storage);
-	r = recvfrom(soc, buf, 511, 0, sfrom, &sfromsiz);
-	if (r < 0) return errno;
+        sfromsiz = sizeof(struct sockaddr_storage);
+        r = recvfrom(soc, buf, 511, 0, sfrom, &sfromsiz);
+        if (r < 0) return errno;
 
-	mbox = strchr(buf, '|');
-	if (mbox) {
-	    *mbox++ = 0;
+        mbox = strchr(buf, '|');
+        if (mbox) {
+            *mbox++ = 0;
         } else {
-	    continue;
+            continue;
         }
 
         handle_request(buf, mbox, sfrom, sfromsiz);
@@ -170,15 +172,15 @@ void service_abort(int error)
 }
 
 int service_main(int argc __attribute__((unused)),
-		 char **argv __attribute__((unused)),
-		 char **envp __attribute__((unused)))
+                 char **argv __attribute__((unused)),
+                 char **envp __attribute__((unused)))
 {
-    int r = 0; 
+    int r = 0;
 
     /* Set namespace */
     if ((r = mboxname_init_namespace(&fud_namespace, 1)) != 0) {
-	syslog(LOG_ERR, "%s", error_message(r));
-	fatal(error_message(r), EC_CONFIG);
+        syslog(LOG_ERR, "%s", error_message(r));
+        fatal(error_message(r), EC_CONFIG);
     }
 
     r = begin_handling();
@@ -196,30 +198,30 @@ static int setsigalrm(int enable)
 
 {
     struct sigaction action;
-    
+
     sigemptyset(&action.sa_mask);
-    
+
     action.sa_flags = 0;
 
     if(enable) {
-	action.sa_handler = cyrus_timeout;
+        action.sa_handler = cyrus_timeout;
     } else {
-	action.sa_handler = SIG_IGN;
+        action.sa_handler = SIG_IGN;
     }
-    
+
     if (sigaction(SIGALRM, &action, NULL) < 0) {
-	syslog(LOG_ERR, "installing SIGALRM handler: sigaction: %m");
-	return -1;
+        syslog(LOG_ERR, "installing SIGALRM handler: sigaction: %m");
+        return -1;
     }
-    
+
     return 0;
 }
 
 
 /* Send a proxy request to the backend, send their reply to sfrom */
 static int do_proxy_request(const char *who, const char *name,
-			    const char *backend_host,
-			    struct sockaddr *sfrom, socklen_t sfromsiz) 
+                            const char *backend_host,
+                            struct sockaddr *sfrom, socklen_t sfromsiz)
 {
     char tmpbuf[1024];
     int replysize;
@@ -240,23 +242,23 @@ static int do_proxy_request(const char *who, const char *name,
     hints.ai_socktype = SOCK_DGRAM;
 
     if (backend_port) {
-	error = getaddrinfo(backend_host, backend_port, &hints, &res0);
+        error = getaddrinfo(backend_host, backend_port, &hints, &res0);
     }
     else {
-	backend_port = "fud";
-	error = getaddrinfo(backend_host, backend_port, &hints, &res0);
-	if (error == EAI_SERVICE) {
-	    backend_port = "4201"; /* default fud port */
-	    error = getaddrinfo(backend_host, backend_port, &hints, &res0);
-	}
+        backend_port = "fud";
+        error = getaddrinfo(backend_host, backend_port, &hints, &res0);
+        if (error == EAI_SERVICE) {
+            backend_port = "4201"; /* default fud port */
+            error = getaddrinfo(backend_host, backend_port, &hints, &res0);
+        }
     }
 
     if (error) {
-	send_reply(sfrom, sfromsiz, REQ_UNK, who, name, 0, 0, 0);
-	r = IMAP_SERVER_UNAVAILABLE;
-	goto done;
+        send_reply(sfrom, sfromsiz, REQ_UNK, who, name, 0, 0, 0);
+        r = IMAP_SERVER_UNAVAILABLE;
+        goto done;
     }
-    
+
     /*
      * XXX: Since UDP is used, we cannot use an IPv6->IPv4 fallback
      * strategy, here.  So, when we can use same address family with
@@ -265,26 +267,26 @@ static int do_proxy_request(const char *who, const char *name,
      */
     csoc = -1;
     for (res = res0; res; res = res->ai_next) {
-	if (res->ai_family == sfrom->sa_family) {
-	    csoc = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
-	    break;
-	}
+        if (res->ai_family == sfrom->sa_family) {
+            csoc = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
+            break;
+        }
     }
     if (csoc < 0) {
-	for (res = res0; res; res = res->ai_next) {
-	    if (res->ai_family != sfrom->sa_family &&
-		(res->ai_family == AF_INET || res->ai_family == AF_INET6)) {
-		csoc = socket(res->ai_family, res->ai_socktype,
-			      res->ai_protocol);
-		break;
-	    }
-	}
+        for (res = res0; res; res = res->ai_next) {
+            if (res->ai_family != sfrom->sa_family &&
+                (res->ai_family == AF_INET || res->ai_family == AF_INET6)) {
+                csoc = socket(res->ai_family, res->ai_socktype,
+                              res->ai_protocol);
+                break;
+            }
+        }
     }
     if (csoc < 0) {
-	send_reply(sfrom, sfromsiz, REQ_UNK, who, name, 0, 0, 0);
-	r = IMAP_SERVER_UNAVAILABLE;
-	freeaddrinfo(res0);
-	goto done;
+        send_reply(sfrom, sfromsiz, REQ_UNK, who, name, 0, 0, 0);
+        r = IMAP_SERVER_UNAVAILABLE;
+        freeaddrinfo(res0);
+        goto done;
     }
 
     cinsiz = res->ai_addrlen;
@@ -299,9 +301,9 @@ static int do_proxy_request(const char *who, const char *name,
     sendto(csoc, tmpbuf, strlen (tmpbuf), 0, cin, cinsiz);
 
     if (setsigalrm(1) < 0) {
-	r = IMAP_SERVER_UNAVAILABLE;
-	send_reply(sfrom, sfromsiz, REQ_UNK, who, name, 0, 0, 0);
-	goto done;
+        r = IMAP_SERVER_UNAVAILABLE;
+        send_reply(sfrom, sfromsiz, REQ_UNK, who, name, 0, 0, 0);
+        goto done;
     }
 
     r = 0;
@@ -316,9 +318,9 @@ static int do_proxy_request(const char *who, const char *name,
     setsigalrm(0);  /* Failure isn't really terrible here */
 
     if (replysize < 1) {
-	r = IMAP_SERVER_UNAVAILABLE;
-	send_reply(sfrom, sfromsiz, REQ_UNK, who, name, 0, 0, 0);
-	goto done;
+        r = IMAP_SERVER_UNAVAILABLE;
+        send_reply(sfrom, sfromsiz, REQ_UNK, who, name, 0, 0, 0);
+        goto done;
     }
 
     /* Send reply back */
@@ -332,7 +334,7 @@ static int do_proxy_request(const char *who, const char *name,
 }
 
 static int handle_request(const char *who, const char *name,
-			  struct sockaddr *sfrom, socklen_t sfromsiz)
+                          struct sockaddr *sfrom, socklen_t sfromsiz)
 {
     int r;
     struct mailbox *mailbox = NULL;
@@ -340,7 +342,6 @@ static int handle_request(const char *who, const char *name,
     time_t lastarrived;
     unsigned recentuid;
     unsigned numrecent;
-    char mboxname[MAX_MAILBOX_BUFFER];
     mbentry_t *mbentry = NULL;
     struct auth_state *mystate;
     int internalseen;
@@ -349,123 +350,125 @@ static int handle_request(const char *who, const char *name,
     lastread = 0;
     lastarrived = 0;
 
-    r = (*fud_namespace.mboxname_tointernal)(&fud_namespace,name,who,mboxname);
-    if (r) return r; 
+    char *intname = mboxname_from_external(name, &fud_namespace, who);
 
-    r = mboxlist_lookup(mboxname, &mbentry, NULL);
+    r = mboxlist_lookup(intname, &mbentry, NULL);
     if (r || mbentry->mbtype & MBTYPE_RESERVE) {
-	send_reply(sfrom, sfromsiz, REQ_UNK, who, name, 0, 0, 0);
-	mboxlist_entry_free(&mbentry);
-	return r;
+        send_reply(sfrom, sfromsiz, REQ_UNK, who, name, 0, 0, 0);
+        mboxlist_entry_free(&mbentry);
+        free(intname);
+        return r;
     }
 
     mystate = auth_newstate("anonymous");
 
     if (mbentry->mbtype & MBTYPE_REMOTE) {
-	/* Check the ACL */
-	if(cyrus_acl_myrights(mystate, mbentry->acl) & ACL_USER0) {
-	    /* We want to proxy this one */
-	    auth_freestate(mystate);
-	    r = do_proxy_request(who, name, mbentry->server, sfrom, sfromsiz);
-	    mboxlist_entry_free(&mbentry);
-	    return r;
-	} else {
-	    /* Permission Denied */
-	    auth_freestate(mystate);
-	    mboxlist_entry_free(&mbentry);
-	    send_reply(sfrom, sfromsiz, REQ_DENY, who, name, 0, 0, 0);
-	    return 0;
-	}
+        /* Check the ACL */
+        if(cyrus_acl_myrights(mystate, mbentry->acl) & ACL_USER0) {
+            /* We want to proxy this one */
+            auth_freestate(mystate);
+            r = do_proxy_request(who, name, mbentry->server, sfrom, sfromsiz);
+            mboxlist_entry_free(&mbentry);
+            free(intname);
+            return r;
+        } else {
+            /* Permission Denied */
+            auth_freestate(mystate);
+            mboxlist_entry_free(&mbentry);
+            free(intname);
+            send_reply(sfrom, sfromsiz, REQ_DENY, who, name, 0, 0, 0);
+            return 0;
+        }
     }
 
     mboxlist_entry_free(&mbentry);
 
     /*
-     * Open/lock header 
+     * Open/lock header
      */
-    r = mailbox_open_irl(mboxname, &mailbox);
+    r = mailbox_open_irl(intname, &mailbox);
     if (r) {
         send_reply(sfrom, sfromsiz, REQ_UNK, who, name, 0, 0, 0);
-	return r; 
+        return r;
     }
 
-    if (mboxname_isusermailbox(mboxname, 0)) {
-	int myrights = cyrus_acl_myrights(mystate, mailbox->acl);
-	if (!(myrights & ACL_USER0)) {
-	    auth_freestate(mystate);
-	    mailbox_close(&mailbox);
-	    send_reply(sfrom, sfromsiz, REQ_DENY, who, name, 0, 0, 0);
-	    return 0;
-	}
+    if (mboxname_isusermailbox(intname, 0)) {
+        int myrights = cyrus_acl_myrights(mystate, mailbox->acl);
+        if (!(myrights & ACL_USER0)) {
+            auth_freestate(mystate);
+            mailbox_close(&mailbox);
+            free(intname);
+            send_reply(sfrom, sfromsiz, REQ_DENY, who, name, 0, 0, 0);
+            return 0;
+        }
     }
     auth_freestate(mystate);
-   
+
     internalseen = mailbox_internal_seen(mailbox, who);
     if (internalseen) {
-	lastread = mailbox->i.recenttime;
-	recentuid = mailbox->i.recentuid;
+        lastread = mailbox->i.recenttime;
+        recentuid = mailbox->i.recentuid;
     } else {
-	struct seen *seendb = NULL;
-	struct seendata sd = SEENDATA_INITIALIZER;
-	r = seen_open(who, 0, &seendb);
-	if (!r) r = seen_read(seendb, mailbox->uniqueid, &sd);
-	seen_close(&seendb);
+        struct seen *seendb = NULL;
+        struct seendata sd = SEENDATA_INITIALIZER;
+        r = seen_open(who, 0, &seendb);
+        if (!r) r = seen_read(seendb, mailbox->uniqueid, &sd);
+        seen_close(&seendb);
 
-	if (r) {
-	    /* Fake Data -- couldn't open seen database */
-	    lastread = 0;
-	    recentuid = 0;
-	}
-	else {
-	    lastread = sd.lastread;
-	    recentuid = sd.lastuid;
-	    seen_freedata(&sd);
-	}
+        if (r) {
+            /* Fake Data -- couldn't open seen database */
+            lastread = 0;
+            recentuid = 0;
+        }
+        else {
+            lastread = sd.lastread;
+            recentuid = sd.lastuid;
+            seen_freedata(&sd);
+        }
     }
 
     lastarrived = mailbox->i.last_appenddate;
     {
-	struct index_record record;
-	uint32_t recno;
-	for (recno = 1; recno <= mailbox->i.num_records; recno++) {
-	    if (mailbox_read_index_record(mailbox, recno, &record))
-		continue;
-	    if (record.system_flags & FLAG_EXPUNGED)
-		continue;
-	    if (record.uid > recentuid) numrecent++;
-	}
+        struct mailbox_iter *iter = mailbox_iter_init(mailbox, 0, ITER_SKIP_EXPUNGED);
+        const message_t *msg;
+        while ((msg = mailbox_iter_step(iter))) {
+            const struct index_record *record = msg_record(msg);
+            if (record->uid > recentuid) numrecent++;
+        }
+        mailbox_iter_done(&iter);
     }
 
     mailbox_close(&mailbox);
+    free(intname);
 
     send_reply(sfrom, sfromsiz, REQ_OK, who, name, numrecent,
-	       lastread, lastarrived);
-    
+               lastread, lastarrived);
+
     return 0;
 }
 
 static void
 send_reply(struct sockaddr *sfrom, socklen_t sfromsiz,
-	   int status, const char *user, const char *mbox,
-	   int numrecent, time_t lastread, time_t lastarrived)
+           int status, const char *user, const char *mbox,
+           int numrecent, time_t lastread, time_t lastarrived)
 {
     char buf[MAX_MAILBOX_PATH + 16 + 9];
     int siz;
 
     switch(status) {
         case REQ_DENY:
-            sendto(soc, "PERMDENY", 9, 0, sfrom, sfromsiz);       
+            sendto(soc, "PERMDENY", 9, 0, sfrom, sfromsiz);
             break;
         case REQ_OK:
             siz = snprintf(buf, sizeof(buf), "%s|%s|%d|%u|%u",
-			   user, mbox, numrecent,
-			   (uint32_t)lastread, (uint32_t)lastarrived);
-            sendto(soc, buf, siz, 0, sfrom, sfromsiz);       
+                           user, mbox, numrecent,
+                           (uint32_t)lastread, (uint32_t)lastarrived);
+            sendto(soc, buf, siz, 0, sfrom, sfromsiz);
             break;
         case REQ_UNK:
             sendto(soc,"UNKNOWN", 8, 0, sfrom, sfromsiz);
             break;
-    } 
+    }
 }
 
 EXPORTED void fatal(const char* s, int code)
@@ -473,7 +476,7 @@ EXPORTED void fatal(const char* s, int code)
     static int recurse_code = 0;
     if (recurse_code) {
         /* We were called recursively. Just give up */
-	exit(code);
+        exit(code);
     }
     recurse_code = code;
     syslog(LOG_ERR, "Fatal error: %s", s);
