@@ -58,7 +58,7 @@
 
 /* create telemetry log; return fd of log */
 EXPORTED int telemetry_log(const char *userid, struct protstream *pin,
-		  struct protstream *pout, int usetimestamp)
+                  struct protstream *pout, int usetimestamp)
 {
     char buf[1024];
     char buf2[1024];
@@ -66,34 +66,41 @@ EXPORTED int telemetry_log(const char *userid, struct protstream *pin,
     time_t now;
     int r;
 
-    if(usetimestamp) {
-	struct timeval tv;
+    if (usetimestamp) {
+        struct timeval tv;
 
-	gettimeofday(&tv, NULL);
+        gettimeofday(&tv, NULL);
 
-	/* use sec.clocks */
-	snprintf(buf, sizeof(buf), "%s%s%s/%s-%lu.%lu",
-		 config_dir, FNAME_LOGDIR, userid, config_ident,
-		 (unsigned long)tv.tv_sec, (unsigned long)tv.tv_usec);
-    } else {
-	/* use pid */
-	snprintf(buf, sizeof(buf), "%s%s%s/%s-%lu",
-		 config_dir, FNAME_LOGDIR, userid, config_ident,
-		 (unsigned long) getpid());
+        /* use sec.clocks */
+        snprintf(buf, sizeof(buf), "%s%s%s/%s-%lu.%lu",
+                 config_dir, FNAME_LOGDIR, userid, config_ident,
+                 (unsigned long)tv.tv_sec, (unsigned long)tv.tv_usec);
+    }
+    else if (config_getswitch(IMAPOPT_TELEMETRY_BYSESSIONID)) {
+        const char *sid = session_id();
+        /* use pid */
+        snprintf(buf, sizeof(buf), "%s%s%s/%s-%s",
+                 config_dir, FNAME_LOGDIR, userid, config_ident, sid);
+    }
+    else {
+        /* use pid */
+        snprintf(buf, sizeof(buf), "%s%s%s/%s-%lu",
+                 config_dir, FNAME_LOGDIR, userid, config_ident,
+                 (unsigned long) getpid());
     }
 
     fd = open(buf, O_CREAT | O_APPEND | O_WRONLY, 0644);
 
     if (fd != -1) {
-	now = time(NULL);
-	snprintf(buf2, sizeof(buf2), "---------- %s %s\n",
-		 userid, ctime(&now));
-	r = write(fd, buf2, strlen(buf2));
-	if (r < 0)
-	    syslog(LOG_ERR, "IOERROR: unable to write to telemetry log %s: %m", buf);
+        now = time(NULL);
+        snprintf(buf2, sizeof(buf2), "---------- %s %s\n",
+                 userid, ctime(&now));
+        r = write(fd, buf2, strlen(buf2));
+        if (r < 0)
+            syslog(LOG_ERR, "IOERROR: unable to write to telemetry log %s: %m", buf);
 
-	prot_setlog(pin, fd);
-	prot_setlog(pout, fd);
+        prot_setlog(pin, fd);
+        prot_setlog(pout, fd);
     }
 
     return fd;
@@ -101,39 +108,39 @@ EXPORTED int telemetry_log(const char *userid, struct protstream *pin,
 
 EXPORTED void telemetry_rusage(char *userid)
 {
-    static struct rusage	previous;
-    struct rusage		current;
-    struct timeval		sys, user;
+    static struct rusage        previous;
+    struct rusage               current;
+    struct timeval              sys, user;
 
     if (userid && *userid) {
-	if (getrusage(RUSAGE_SELF, &current) != 0) {
-	    syslog(LOG_ERR, "getrusage: %s", userid);
-	    return;
-	}
+        if (getrusage(RUSAGE_SELF, &current) != 0) {
+            syslog(LOG_ERR, "getrusage: %s", userid);
+            return;
+        }
 
-	user.tv_sec = current.ru_utime.tv_sec - previous.ru_utime.tv_sec;
-	user.tv_usec = current.ru_utime.tv_usec - previous.ru_utime.tv_usec;
-	if (user.tv_usec < 0) {
-	    user.tv_sec--;
-	    user.tv_usec += 1000000;
-	}
+        user.tv_sec = current.ru_utime.tv_sec - previous.ru_utime.tv_sec;
+        user.tv_usec = current.ru_utime.tv_usec - previous.ru_utime.tv_usec;
+        if (user.tv_usec < 0) {
+            user.tv_sec--;
+            user.tv_usec += 1000000;
+        }
 
-	sys.tv_sec = current.ru_stime.tv_sec - previous.ru_stime.tv_sec;
-	sys.tv_usec = current.ru_stime.tv_usec - previous.ru_stime.tv_usec;
-	if (sys.tv_usec < 0) {
-	    sys.tv_sec--;
-	    sys.tv_usec += 1000000;
-	}
+        sys.tv_sec = current.ru_stime.tv_sec - previous.ru_stime.tv_sec;
+        sys.tv_usec = current.ru_stime.tv_usec - previous.ru_stime.tv_usec;
+        if (sys.tv_usec < 0) {
+            sys.tv_sec--;
+            sys.tv_usec += 1000000;
+        }
 
-	/*
-	 * Some systems provide significantly more data, but POSIX
-	 * guarantees user & sys CPU time.
-	 */
-	syslog(LOG_NOTICE, "USAGE %s user: %lu.%.6d sys: %lu.%.6d", userid,
-	       (unsigned long)user.tv_sec, (int)user.tv_usec,
-	       (unsigned long)sys.tv_sec, (int)sys.tv_usec);
+        /*
+         * Some systems provide significantly more data, but POSIX
+         * guarantees user & sys CPU time.
+         */
+        syslog(LOG_NOTICE, "USAGE %s user: %lu.%.6d sys: %lu.%.6d", userid,
+               (unsigned long)user.tv_sec, (int)user.tv_usec,
+               (unsigned long)sys.tv_sec, (int)sys.tv_usec);
 
-	previous = current;
+        previous = current;
     }
 
     return;
