@@ -47,17 +47,17 @@ EXPORTED hash_table *construct_hash_table(hash_table *table, size_t size, int us
 
       /* Allocate the table -- different for using memory pools and not */
       if(use_mpool) {
-	  /* Allocate an initial memory pool for 32 byte keys + the hash table
-	   * + the buckets themselves */
-	  table->pool =
-	      new_mpool(size * (32 + sizeof(bucket*) + sizeof(bucket)));
-	  table->table =
-	      (bucket **)mpool_malloc(table->pool,sizeof(bucket *) * size);
+          /* Allocate an initial memory pool for 32 byte keys + the hash table
+           * + the buckets themselves */
+          table->pool =
+              new_mpool(size * (32 + sizeof(bucket*) + sizeof(bucket)));
+          table->table =
+              (bucket **)mpool_malloc(table->pool,sizeof(bucket *) * size);
       } else {
-	  table->pool = NULL;
-	  table->table = xmalloc(sizeof(bucket *) * size);
+          table->pool = NULL;
+          table->table = xmalloc(sizeof(bucket *) * size);
       }
-       
+
       /* Allocate the table and initilize it */
       memset(table->table, 0, sizeof(bucket *) * size);
 
@@ -83,17 +83,17 @@ EXPORTED void *hash_insert(const char *key, void *data, hash_table *table)
       */
       if (!((table->table)[val]))
       {
-	  if(table->pool) {
-	      (table->table)[val] =
-		  (bucket *)mpool_malloc(table->pool, sizeof(bucket));
-	      (table->table)[val] -> key = mpool_strdup(table->pool, key);
-	  } else {
-	      (table->table)[val] = (bucket *)xmalloc(sizeof(bucket));
-	      (table->table)[val] -> key = xstrdup(key);
-	  }
-	  (table->table)[val] -> next = NULL;
-	  (table->table)[val] -> data = data;
-	  return (table->table)[val] -> data;
+          if(table->pool) {
+              (table->table)[val] =
+                  (bucket *)mpool_malloc(table->pool, sizeof(bucket));
+              (table->table)[val] -> key = mpool_strdup(table->pool, key);
+          } else {
+              (table->table)[val] = (bucket *)xmalloc(sizeof(bucket));
+              (table->table)[val] -> key = xstrdup(key);
+          }
+          (table->table)[val] -> next = NULL;
+          (table->table)[val] -> data = data;
+          return (table->table)[val] -> data;
       }
 
       /*
@@ -101,31 +101,31 @@ EXPORTED void *hash_insert(const char *key, void *data, hash_table *table)
       ** has already been inserted, and if so, increment its count.
       */
       for (prev = &((table->table)[val]), ptr=(table->table)[val];
-	   ptr;
-	   prev=&(ptr->next),ptr=ptr->next) {
-	  int cmpresult = strcmp(key,ptr->key);
-	  if (!cmpresult) {
-	      /* Match! Replace this value and return the old */
-	      void *old_data;
-	      
-	      old_data = ptr->data;
-	      ptr -> data = data;
-	      return old_data;
-	  } else if (cmpresult < 0) {
-	      /* The new key is smaller than the current key--
-	       * insert a node and return this data */
-	      if(table->pool) {
-		  newptr = (bucket *)mpool_malloc(table->pool, sizeof(bucket));
-		  newptr->key = mpool_strdup(table->pool, key);
-	      } else {
-		  newptr = (bucket *)xmalloc(sizeof(bucket));
-		  newptr->key = xstrdup(key);
-	      }
-	      newptr->data = data;
-	      newptr->next = ptr;
-	      *prev = newptr;
-	      return data;
-	  }
+           ptr;
+           prev=&(ptr->next),ptr=ptr->next) {
+          int cmpresult = strcmp(key,ptr->key);
+          if (!cmpresult) {
+              /* Match! Replace this value and return the old */
+              void *old_data;
+
+              old_data = ptr->data;
+              ptr -> data = data;
+              return old_data;
+          } else if (cmpresult < 0) {
+              /* The new key is smaller than the current key--
+               * insert a node and return this data */
+              if(table->pool) {
+                  newptr = (bucket *)mpool_malloc(table->pool, sizeof(bucket));
+                  newptr->key = mpool_strdup(table->pool, key);
+              } else {
+                  newptr = (bucket *)xmalloc(sizeof(bucket));
+                  newptr->key = xstrdup(key);
+              }
+              newptr->data = data;
+              newptr->next = ptr;
+              *prev = newptr;
+              return data;
+          }
       }
 
       /*
@@ -133,11 +133,11 @@ EXPORTED void *hash_insert(const char *key, void *data, hash_table *table)
       ** of the list (*prev should be correct)
       */
       if(table->pool) {
-	  newptr=(bucket *)mpool_malloc(table->pool,sizeof(bucket));
-	  newptr->key = mpool_strdup(table->pool,key);
+          newptr=(bucket *)mpool_malloc(table->pool,sizeof(bucket));
+          newptr->key = mpool_strdup(table->pool,key);
       } else {
-	  newptr=(bucket *)xmalloc(sizeof(bucket));
-	  newptr->key = xstrdup(key);
+          newptr=(bucket *)xmalloc(sizeof(bucket));
+          newptr->key = xstrdup(key);
       }
       newptr->data = data;
       newptr->next = NULL;
@@ -161,11 +161,11 @@ EXPORTED void *hash_lookup(const char *key, hash_table *table)
 
       for ( ptr = (table->table)[val];NULL != ptr; ptr = ptr->next )
       {
-	  int cmpresult = strcmp(key, ptr->key);
-	  if (!cmpresult)
-	      return ptr->data;
-	  else if(cmpresult < 0) /* key < ptr->key -- we passed it */
-	      return NULL;
+          int cmpresult = strcmp(key, ptr->key);
+          if (!cmpresult)
+              return ptr->data;
+          else if(cmpresult < 0) /* key < ptr->key -- we passed it */
+              return NULL;
       }
       return NULL;
 }
@@ -197,42 +197,42 @@ EXPORTED void *hash_del(const char *key, hash_table *table)
             NULL != ptr;
             last = ptr, ptr = ptr->next)
       {
-	  int cmpresult = strcmp(key, ptr->key);
-	  if (!cmpresult)
-	  {
-	      if (last != NULL )
-	      {
-		  data = ptr -> data;
-		  last -> next = ptr -> next;
-		  if(!table->pool) {
-		      free(ptr->key);
-		      free(ptr);
-		  }
-		  return data;
-	      }
-	      
-	      /*
-	      ** If 'last' still equals NULL, it means that we need to
-	      ** delete the first node in the list. This simply consists
-	      ** of putting our own 'next' pointer in the array holding
-	      ** the head of the list.  We then dispose of the current
-	      ** node as above.
-	      */
-	      
-	      else
-	      {
-		  data = ptr->data;
-		  (table->table)[val] = ptr->next;
-		  if(!table->pool) {
-		      free(ptr->key);
-		      free(ptr);
-		  }
-		  return data;
-	      }
-	  } else if (cmpresult < 0) {
-	      /* its not here! */
-	      return NULL;
-	  }
+          int cmpresult = strcmp(key, ptr->key);
+          if (!cmpresult)
+          {
+              if (last != NULL )
+              {
+                  data = ptr -> data;
+                  last -> next = ptr -> next;
+                  if(!table->pool) {
+                      free(ptr->key);
+                      free(ptr);
+                  }
+                  return data;
+              }
+
+              /*
+              ** If 'last' still equals NULL, it means that we need to
+              ** delete the first node in the list. This simply consists
+              ** of putting our own 'next' pointer in the array holding
+              ** the head of the list.  We then dispose of the current
+              ** node as above.
+              */
+
+              else
+              {
+                  data = ptr->data;
+                  (table->table)[val] = ptr->next;
+                  if(!table->pool) {
+                      free(ptr->key);
+                      free(ptr);
+                  }
+                  return data;
+              }
+          } else if (cmpresult < 0) {
+              /* its not here! */
+              return NULL;
+          }
       }
 
       /*
@@ -259,29 +259,29 @@ EXPORTED void free_hash_table(hash_table *table, void (*func)(void *))
       /* We also need to traverse this anyway if we aren't using a memory
        * pool */
       if(func || !table->pool) {
-	  for (i=0;i<table->size; i++)
-	  {
-	      ptr = (table->table)[i];
-	      while (ptr)
-	      {
-		  temp = ptr;
-		  ptr = ptr->next;
-		  if (func)
-		      func(temp->data);
-		  if(!table->pool) {
-		      free(temp->key);
-		      free(temp);
-		  }
-	      }
-	  }
+          for (i=0;i<table->size; i++)
+          {
+              ptr = (table->table)[i];
+              while (ptr)
+              {
+                  temp = ptr;
+                  ptr = ptr->next;
+                  if (func)
+                      func(temp->data);
+                  if(!table->pool) {
+                      free(temp->key);
+                      free(temp);
+                  }
+              }
+          }
       }
-      
+
       /* Free the main structures */
       if(table->pool) {
-	  free_mpool(table->pool);
-	  table->pool = NULL;
+          free_mpool(table->pool);
+          table->pool = NULL;
       } else {
-	  free(table->table);
+          free(table->table);
       }
       table->table = NULL;
       table->size = 0;
@@ -293,7 +293,7 @@ EXPORTED void free_hash_table(hash_table *table, void (*func)(void *))
 */
 
 EXPORTED void hash_enumerate(hash_table *table, void (*func)(const char *, void *, void *),
-		    void *rock)
+                    void *rock)
 {
       unsigned i;
       bucket *temp, *temp_next;
@@ -306,10 +306,44 @@ EXPORTED void hash_enumerate(hash_table *table, void (*func)(const char *, void
                         NULL != temp;
                         temp = temp_next)
                   {
-			temp_next = temp->next;
+                        temp_next = temp->next;
                         func(temp -> key, temp->data, rock);
                   }
             }
       }
 }
 
+EXPORTED strarray_t *hash_keys(hash_table *table)
+{
+    unsigned i;
+    bucket *temp;
+
+    strarray_t *sa = strarray_new();
+
+    for (i = 0; i < table->size; i++) {
+        temp = (table->table)[i];
+        while (temp) {
+            strarray_append(sa, temp->key);
+            temp = temp->next;
+        }
+    }
+
+    return sa;
+}
+
+EXPORTED int hash_numrecords(hash_table *table)
+{
+    unsigned i;
+    bucket *temp;
+    int count = 0;
+
+    for (i = 0; i < table->size; i++) {
+        temp = (table->table)[i];
+        while (temp) {
+            count++;
+            temp = temp->next;
+        }
+    }
+
+    return count;
+}
