@@ -78,84 +78,57 @@ EXPORTED int lock_reopen_ex(int fd, const char *filename,
     if (!sbuf) sbuf = &sbufspare;
 
     for (;;) {
-	r = flock(fd, LOCK_EX);
-	if (r == -1) {
-	    if (errno == EINTR) continue;
-	    if (failaction) *failaction = "locking";
-	    return -1;
-	}
+        r = flock(fd, LOCK_EX);
+        if (r == -1) {
+            if (errno == EINTR) continue;
+            if (failaction) *failaction = "locking";
+            return -1;
+        }
 
-	fstat(fd, sbuf);
-	r = stat(filename, &sbuffile);
-	if (r == -1) {
-	    if (failaction) *failaction = "stating";
-	    flock(fd, LOCK_UN);
-	    return -1;
-	}
+        r = fstat(fd, sbuf);
+        if (!r) r = stat(filename, &sbuffile);
+        if (r == -1) {
+            if (failaction) *failaction = "stating";
+            lock_unlock(fd, filename);
+            return -1;
+        }
 
-	if (sbuf->st_ino == sbuffile.st_ino) return 0;
+        if (sbuf->st_ino == sbuffile.st_ino) return 0;
 
         if (changed) *changed = 1;
 
-	newfd = open(filename, O_RDWR);
-	if (newfd == -1) {
-	    if (failaction) *failaction = "opening";
-	    flock(fd, LOCK_UN);
-	    return -1;
-	}
-	dup2(newfd, fd);
-	close(newfd);
+        newfd = open(filename, O_RDWR);
+        if (newfd == -1) {
+            if (failaction) *failaction = "opening";
+            lock_unlock(fd, filename);
+            return -1;
+        }
+        dup2(newfd, fd);
+        close(newfd);
     }
 }
 
 /*
- * Obtain an exclusive lock on 'fd'.
+ * Obtain a lock on 'fd'.  The lock is exclusive if 'exclusive'
+ * is true, otherwise shared.  Normally blocks until a lock is
+ * obtained, but if 'nonblock' is true does not block and instead
+ * fails with errno=EWOUDBLOCK if the lock cannot be obtained.
+ *
  * Returns 0 for success, -1 for failure, with errno set to an
  * appropriate error code.
  */
-EXPORTED int lock_blocking(int fd, const char *filename __attribute__((unused)))
+EXPORTED int lock_setlock(int fd, int exclusive, int nonblock,
+                          const char *filename __attribute__((unused)))
 {
     int r;
+    int op = (exclusive ? LOCK_EX : LOCK_SH);
+    if (nonblock) op |= LOCK_NB;
 
     for (;;) {
-	r = flock(fd, LOCK_EX);
-	if (r != -1) return 0;
-	if (errno == EINTR) continue;
-	return -1;
-    }
-}
-
-/*
- * Obtain a shared lock on 'fd'.
- * Returns 0 for success, -1 for failure, with errno set to an
- * appropriate error code.
- */
-EXPORTED int lock_shared(int fd, const char *filename __attribute__((unused)))
-{
-    int r;
-
-    for (;;) {
-	r = flock(fd, LOCK_SH);
-	if (r != -1) return 0;
-	if (errno == EINTR) continue;
-	return -1;
-    }
-}
-
-/*
- * Attempt to get an exclusive lock on 'fd' without blocking.
- * Returns 0 for success, -1 for failure, with errno set to an
- * appropriate error code.
- */
-EXPORTED int lock_nonblocking(int fd, const char *filename __attribute__((unused)))
-{
-    int r;
-
-    for (;;) {
-	r = flock(fd, LOCK_EX|LOCK_NB);
-	if (r != -1) return 0;
-	if (errno == EINTR) continue;
-	return -1;
+        r = flock(fd, op);
+        if (r != -1) return 0;
+        if (errno == EINTR) continue;
+        return -1;
     }
 }
 
@@ -167,11 +140,11 @@ EXPORTED int lock_unlock(int fd, const char *filename __attribute__((unused)))
     int r;
 
     for (;;) {
-	r = flock(fd, LOCK_UN);
-	if (r != -1) return 0;
-	if (errno == EINTR) continue;
-	/* xxx help! */
-	return -1;
+        r = flock(fd, LOCK_UN);
+        if (r != -1) return 0;
+        if (errno == EINTR) continue;
+        /* xxx help! */
+        return -1;
     }
 }
 
