@@ -42,7 +42,7 @@
 /* This program purges old entries from the database. It holds an exclusive
  * lock throughout the process.
  *
- * NOTE: by adding the alt_file flag, we let exit() handle the cleanup of 
+ * NOTE: by adding the alt_file flag, we let exit() handle the cleanup of
  *       the lock file's fd. That's bad in principal but not in practice. We do
  *       to make the code easier to read.
  */
@@ -68,32 +68,31 @@
 #include "imap/global.h"
 #include "libconfig.h"
 #include "xmalloc.h"
-#include "xversion.h"
 
 /* global */
 time_t timenow;
 time_t expire_time = (3*60*60); /* 3 Hours */
 
 static int expire_p(void *rockp __attribute__((unused)),
-		    const char *key __attribute__((unused)),
-		    size_t keylen __attribute__((unused)),
-		    const char *data,
-		    size_t datalen __attribute__((unused)))
+                    const char *key __attribute__((unused)),
+                    size_t keylen __attribute__((unused)),
+                    const char *data,
+                    size_t datalen __attribute__((unused)))
 {
     struct auth_state *authstate = (struct auth_state *)data;
     if (authstate->mark + expire_time < timenow) {
-	return 1;
+        return 1;
     }
     return 0; /* skip this one */
 }
 
 static int expire_cb(void *rockp,
-		     const char *key, size_t keylen,
-		     const char *data __attribute__((unused)),
-		     size_t datalen __attribute__((unused)))
+                     const char *key, size_t keylen,
+                     const char *data __attribute__((unused)),
+                     size_t datalen __attribute__((unused)))
 {
     /* We only get called when we want to delete it */
-    syslog(LOG_DEBUG, "deleteing entry for %s", key);
+    syslog(LOG_DEBUG, "deleting entry for %s", key);
 
     /* xxx maybe we should use transactions for this */
     cyrusdb_delete((struct db *)rockp, key, keylen, NULL, 0);
@@ -103,61 +102,55 @@ static int expire_cb(void *rockp,
 int main(int argc, char *argv[])
 {
     struct db *ptdb;
+    char fnamebuf[1024];
     extern char *optarg;
     int opt;
     int r;
-    const char *fname;
-    char *alt_config = NULL, *tofree = NULL;
+    char *alt_config = NULL;
 
     if ((geteuid()) == 0 && (become_cyrus(/*is_master*/0) != 0)) {
-	fatal("must run as the Cyrus user", EC_USAGE);
+        fatal("must run as the Cyrus user", EC_USAGE);
     }
-    
+
     openlog("ptexpire", LOG_PID, SYSLOG_FACILITY);
 
     while ((opt = getopt(argc, argv, "C:E:")) != EOF) {
-	switch (opt) {
-	case 'C': /* alt config file */
-	    alt_config = optarg;
-	    break;
-	case 'E':
-	    expire_time = atoi(optarg);
-	    break;
-	default:
-	    fprintf(stderr,"usage: [-C filename] [-E time]"
-		    "\n\t-C <filename>\tAlternate Config File"
-		    "\n\t-E <seconds>\tExpiration time"
-		    "\n");
-	    syslog(LOG_ERR, "Invalid command line option");
-	    exit(-1);
-	    break;
-	    /* just pass through */
-	}
+        switch (opt) {
+        case 'C': /* alt config file */
+            alt_config = optarg;
+            break;
+        case 'E':
+            expire_time = atoi(optarg);
+            break;
+        default:
+            fprintf(stderr,"usage: [-C filename] [-E time]"
+                    "\n\t-C <filename>\tAlternate Config File"
+                    "\n\t-E <seconds>\tExpiration time"
+                    "\n");
+            syslog(LOG_ERR, "Invalid command line option");
+            exit(-1);
+            break;
+            /* just pass through */
+        }
     }
 
     cyrus_init(alt_config, "ptexpire", 0, 0);
 
     timenow = time(0);
     syslog(LOG_INFO, "Expiring entries older than %d seconds (currently %d)",
-	   (int)expire_time, (int)timenow);
-    syslog(LOG_DEBUG, "%s", "ptexpire.c,v " _CYRUS_VERSION " " CYRUS_GITVERSION);
-    
+           (int)expire_time, (int)timenow);
+    syslog(LOG_DEBUG, "ptexpire.c %s", PACKAGE_VERSION);
+
     /* open database */
-    fname = config_getstring(IMAPOPT_PTSCACHE_DB_PATH);
-    if (!fname) {
-        tofree = strconcat(config_dir, PTS_DBFIL, NULL);
-        fname = tofree;
-    }
-
-    r = cyrusdb_open(config_ptscache_db, fname, CYRUSDB_CREATE, &ptdb);
+    strcpy(fnamebuf, config_dir);
+    strcat(fnamebuf, PTS_DBFIL);
+    r = cyrusdb_open(config_ptscache_db, fnamebuf, CYRUSDB_CREATE, &ptdb);
     if(r != CYRUSDB_OK) {
-        syslog(LOG_ERR, "error opening %s (%s)", fname,
-	       cyrusdb_strerror(r));
-	exit(1);
+        syslog(LOG_ERR, "error opening %s (%s)", fnamebuf,
+               cyrusdb_strerror(r));
+        exit(1);
     }
 
-    if (tofree) free(tofree);
-
     /* iterate through db, wiping expired entries */
     cyrusdb_foreach(ptdb, "", 0, expire_p, expire_cb, ptdb, NULL);
 
@@ -167,4 +160,4 @@ int main(int argc, char *argv[])
 
     syslog(LOG_INFO, "finished");
     return 0;
-}      
+}
