@@ -63,7 +63,6 @@
 #include "retry.h"
 #include "xmalloc.h"
 #include "ptloader.h"
-#include "xversion.h"
 
 struct pts_module *pts_modules[] = {
 #ifdef HAVE_LDAP
@@ -81,55 +80,57 @@ static struct pts_module *pts_fromname()
     int i;
     const char *name = config_getstring(IMAPOPT_PTS_MODULE);
     static struct pts_module *pts = NULL;
-    
+
     if (pts)
         return pts;
-    
+
     for (i = 0; pts_modules[i]; i++) {
-	if (!strcmp(pts_modules[i]->name, name)) {
-	    pts = pts_modules[i]; break;
-	}
+        if (!strcmp(pts_modules[i]->name, name)) {
+            pts = pts_modules[i]; break;
+        }
     }
 
     if (!pts) {
-	char errbuf[1024];
-	snprintf(errbuf, sizeof(errbuf),
-		 "PTS module %s not supported", name);
-	fatal(errbuf, EC_CONFIG);
+        char errbuf[1024];
+        snprintf(errbuf, sizeof(errbuf),
+                 "PTS module %s not supported", name);
+        fatal(errbuf, EC_CONFIG);
     }
-    
+
     return pts;
 }
 
 void ptsmodule_init(void)
 {
     struct pts_module *pts = pts_fromname();
-    
+
     pts->init();
 }
 
 struct auth_state *ptsmodule_make_authstate(const char *identifier,
-					    size_t size,
-					    const char **reply, int *dsize)
+                                            size_t size,
+                                            const char **reply, int *dsize)
 {
     struct pts_module *pts = pts_fromname();
-    
+
     return pts->make_authstate(identifier, size, reply, dsize);
 }
 
 /* config.c info (libimap) */
 const int config_need_data = 0;
 
+/* Globals */
+#define DB (config_ptscache_db)
+
 static char ptclient_debug = 0;
 struct db *ptsdb = NULL;
-  
+
 int service_init(int argc, char *argv[], char **envp __attribute__((unused)))
 {
     int r;
     int opt;
+    char fnamebuf[1024];
     extern char *optarg;
-    const char *fname;
-    char *tofree = NULL;
 
     if (geteuid() == 0) fatal("must run as the Cyrus user", EC_USAGE);
     setproctitle_init(argc, argv, envp);
@@ -137,38 +138,32 @@ int service_init(int argc, char *argv[], char **envp __attribute__((unused)))
     /* set signal handlers */
     signal(SIGPIPE, SIG_IGN);
 
-    syslog(LOG_NOTICE, "starting: ptloader.c,v " _CYRUS_VERSION " " CYRUS_GITVERSION);
+    syslog(LOG_NOTICE, "starting: ptloader.c %s", PACKAGE_VERSION);
 
     while ((opt = getopt(argc, argv, "d:")) != EOF) {
-	switch (opt) {
-	case 'd':
-	    ptclient_debug = atoi(optarg);
-	    if (ptclient_debug < 1) {
-		ptclient_debug = 1;
-	    }
-	    break;
-	default:
+        switch (opt) {
+        case 'd':
+            ptclient_debug = atoi(optarg);
+            if (ptclient_debug < 1) {
+                ptclient_debug = 1;
+            }
+            break;
+        default:
             syslog(LOG_ERR, "invalid command line option specified");
-	    break;
-	    /* just pass through */
-	}
+            break;
+            /* just pass through */
+        }
     }
 
-    fname = config_getstring(IMAPOPT_PTSCACHE_DB_PATH);
-    if (!fname) {
-        tofree = strconcat(config_dir, PTS_DBFIL, NULL);
-        fname = tofree;
-    }
-
-    r = cyrusdb_open(config_ptscache_db, fname, CYRUSDB_CREATE, &ptsdb);
+    strcpy(fnamebuf, config_dir);
+    strcat(fnamebuf, PTS_DBFIL);
+    r = cyrusdb_open(DB, fnamebuf, CYRUSDB_CREATE, &ptsdb);
     if (r != 0) {
-        syslog(LOG_ERR, "DBERROR: opening %s: %s", fname,
-	       cyrusdb_strerror(ret));
-	fatal("can't read pts database", EC_TEMPFAIL);
+        syslog(LOG_ERR, "DBERROR: opening %s: %s", fnamebuf,
+               cyrusdb_strerror(ret));
+        fatal("can't read pts database", EC_TEMPFAIL);
     }
 
-    if (tofree) free(tofree);
-
     ptsmodule_init();
 
     return 0;
@@ -181,8 +176,8 @@ void service_abort(int error)
 
     r = cyrusdb_close(ptsdb);
     if (r) {
-	syslog(LOG_ERR, "DBERROR: error closing ptsdb: %s",
-	       cyrusdb_strerror(r));
+        syslog(LOG_ERR, "DBERROR: error closing ptsdb: %s",
+               cyrusdb_strerror(r));
     }
 
     cyrusdb_done();
@@ -193,8 +188,8 @@ void service_abort(int error)
 /* we're a 'threaded' service, but since we never fork or create any
    threads, we're just one-person-at-a-time based */
 int service_main_fd(int c, int argc __attribute__((unused)),
-		    char **argv __attribute__((unused)),
-		    char **envp __attribute__((unused)))
+                    char **argv __attribute__((unused)),
+                    char **envp __attribute__((unused)))
 {
     const char *reply = NULL;
     char user[PTS_DB_KEYSIZE];
@@ -210,10 +205,10 @@ int service_main_fd(int c, int argc __attribute__((unused)),
     }
 
     if (size > PTS_DB_KEYSIZE)  {
-	syslog(LOG_ERR, "size sent %d is greater than buffer size %d", 
-	       (int)size, PTS_DB_KEYSIZE);
-	reply = "Error: invalid request size";
-	goto sendreply;
+        syslog(LOG_ERR, "size sent %d is greater than buffer size %d",
+               (int)size, PTS_DB_KEYSIZE);
+        reply = "Error: invalid request size";
+        goto sendreply;
     }
 
     if (size == 0) {
@@ -224,35 +219,35 @@ int service_main_fd(int c, int argc __attribute__((unused)),
 
     memset(&user, 0, sizeof(user));
     if (read(c, &user, size) < 0) {
-	syslog(LOG_ERR, "socket(user; size = %d): %m", (int)size);
+        syslog(LOG_ERR, "socket(user; size = %d): %m", (int)size);
         reply = "Error reading request (user)";
         goto sendreply;
     }
 
     if (ptclient_debug) {
-	syslog(LOG_DEBUG, "user %s", user);
+        syslog(LOG_DEBUG, "user %s", user);
     }
 
     newstate = ptsmodule_make_authstate(user, size, &reply, &dsize);
 
     if(newstate) {
-	/* Success! */
-	rc = cyrusdb_store(ptsdb, user, size, (void *)newstate, dsize, NULL);
-	(void)rc;
+        /* Success! */
+        rc = cyrusdb_store(ptsdb, user, size, (void *)newstate, dsize, NULL);
+        (void)rc;
         free(newstate);
-	
-	/* and we're done */
-	reply = "OK";
+
+        /* and we're done */
+        reply = "OK";
     } else {
         /* Failure */
-	if ( reply == NULL ) {
-	    reply = "Error making authstate";
-	}
+        if ( reply == NULL ) {
+            reply = "Error making authstate";
+        }
     }
 
  sendreply:
     if (retry_write(c, reply, strlen(reply) + 1) <0) {
-	syslog(LOG_WARNING, "retry_write: %m");
+        syslog(LOG_WARNING, "retry_write: %m");
     }
     close(c);
 
@@ -260,7 +255,7 @@ int service_main_fd(int c, int argc __attribute__((unused)),
 }
 
 /* we need to have this function here 'cause libcyrus.la
- * makes calls to this function. 
+ * makes calls to this function.
  */
 EXPORTED void fatal(const char *msg, int exitcode)
 {
